#!/usr/bin/perl

# Try to keep the programming style good ;-)

use strict;
use warnings;

# Some modules we need for password-related Stuff

use Crypt::PasswdMD5;
use Crypt::RandPasswd;

# We want to parse commandline arguments in a comfortable way

use Getopt::Std;

# Parse the commandline arguments and fall back to default values if
# neccessary

my %opts;
getopts('b:g:i:l:S:o:m:x:e:hrsd', \%opts);

if ( $opts{h} ) { die "usage: $0 -i [input file] -g [group] -b [base directory for \$HOME] -s -S [smbpasswd binary] -o [output file] -r -m [minimum password length] -x [maximum password length]\n";}

if ( $> != 0 ) {die "You must be root to run this.\n";}

if ( ! scalar %opts ) {die "usage: $0 -i [input file] -g [group] -b [base directory for \$HOME] -s -S [smbpasswd binary] -o [output file] -r -m [minimum password length] -x [maximum password length]\n";}

my $userliste = "userlist";
if ( $opts{i} ) { $userliste = $opts{i}; }

my $group = "schueler";
if ( $opts{g} ) { $group = $opts{g}; }

if ( ! getgrnam $group ) { die "Group $group doesn't exist. Please create it or specify an existing group.\n" }

my $homebase = "/home/$group";
if ( $opts{b} ) { $homebase = $opts{b}; }

my $smbpass = "/usr/local/bin/smbpasswd";
if ( $opts{S} ) { $smbpass = $opts{S}; }

my $outfile = "useraccounts";
if ( $opts{o} ) { $outfile = $opts{o}; }

my $passtype = "letters";
if ( $opts{r} ) { $passtype = "chars"; }

my $minlength;
if ( $opts{m} ) { $minlength = $opts{m}; }
else	{
	if ( $opts{r} ) { $minlength = 10; }
	else { $minlength = 8; }
	}

my $maxlength;
if ( $opts{x} ) { $maxlength = $opts{x}; }
else	{
	if ( $opts{r} ) { $maxlength = 10; }
	else { $maxlength = 8; }
	}

print "Group: $group\n";
print "Base directory: $homebase\n";
print "Userliste: $userliste\n\n";

# Determine the system we run on

my $uname;
if ( ! stat "/usr/bin/uname" ) { $uname = "/bin/uname"; }
else { $uname = "/usr/bin/uname" }

my $os = `$uname`;
chomp $os;

# Create the filehandle to read the userlist from or read from STDIN if
# $opts{i} is "-"

my $handle;
if ( $opts{i} eq "-" ) { $handle ="STDIN"; }
else
	{
	$handle = "LISTE";
	open LISTE,$userliste or die "$userliste: No such file or directory.\n";
	}

# Do some deep magic to determine the school year.

# First get the raw time
my @time = localtime;

# Extract the relevant parts

my $month = $time[4];
my $year = $time[5];

# Convert the year to 2-digit format

my $year1 = sprintf("%02d", $year % 100);

# Now it's time for the arcane part...

my $schoolyear;
if ($month le 7) {
	my $year2=$year1 - 1;
	$schoolyear = "0$year2/$year1";
}
else {
	my $year2=$year1 + 1;
	$schoolyear = "$year1/0$year2";
}

# Make -s implicit if -S is given

if ( $opts{S} and ! $opts{s} )
	{
	$opts{s} = 1;
	}

# Check for smbpasswd(8) if -s is given

if ( $opts{s} ) 
	{
	if ( ! -x $smbpass ) { die "$smbpass doesn't exist or isn't executable. Please specify.\n"; }
	}



# Extract the account data from the list of usernames

while (my $line = <$handle>)
{
	# Parse the userlist and put it's elements in variables
	my ($name, $realname, $class) = split(/\t/, $line);	

	# Remove the newline left over from parsing the input file

	chomp $class;

	# First check if the username exists. If it exists generate two
	# random letters and attach them to the name. Repeat this
	# procedure if the changed name exists already.

	if ( $opts{d} ) { dupe_engine(); }
	else { dupe_log(); }

	sub dupe_engine
		{
		if ( getpwnam $name ) 
			{
			print "User $name already in passwd database.\n";
			unless ( ! getpwnam $name )
				{
				my $randomizer = Crypt::RandPasswd->word(1,2);
				$name .= $randomizer;
				print "New Name: $name\n";
				}
			}
		}
	sub dupe_log
		{
		if ( getpwnam $name )
			{
			my $errlog = "failed_users";
			if ( $opts{e} ) { $errlog = $opts{e}; }
			print "User $name already in passwd database.";
			open DUPELOG,">> $errlog";
			print DUPELOG "User: $name Realname: $realname Class: $class\n";
			close DUPELOG;
			}
		}
	
	# Compose the GECOS field for /etc/passwd
	my $gecos="$realname;$class;$schoolyear";

	# Generate a password for the user

	my $clearpass = Crypt::RandPasswd->$passtype( $minlength, $maxlength );

	my $passwdhash = unix_md5_crypt ($clearpass, '$1$');

	# Create an Unix account
	# First determine which sytem we run on and which commands we
	# have to use

	if ( $os eq "Linux" )
		{
		# useradd(8) is braindead because it won't let me
		# specify a base directory for $HOME
		my $home = "$homebase/$name";
		if ( $home =~ /\/{2,}/ ) { $home =~ s/\/{2,}/\//g; }

		system "/usr/sbin/useradd -m -c '$gecos' -g $group -d $home $name";
		open PASSPIPE,"| /usr/sbin/chpasswd -e";
		print PASSPIPE "$name:$passwdhash\n";
		close PASSPIPE;
		}

	if ( $os eq "FreeBSD" )
		{
		open PWPIPE, "| /usr/sbin/pw useradd -n $name -m -c '$gecos' -g $group -b $homebase -h 0";
		print PWPIPE "$clearpass\n";
		close PWPIPE;
		}

	# Use smbpasswd(8) to create an smbpasswd entry
	# Yes this is ugly...and so is smbpasswd's non-interactive
	# mode...

	# Compose the complete smbpasswd command

	if ( $opts{s} ) { smb_create(); }

	sub smb_create 
		{
		my $smbpasswd = "$smbpass -s -a $name";
		open SMBPIPE,"| $smbpasswd";
		print SMBPIPE "$clearpass\n$clearpass\n";
		close SMBPIPE;
		}

	# Write the account data to a file

	open OUTFILE,">>$outfile";
	print OUTFILE "Username: $name Password: $clearpass Name: $realname Klasse: $class\n";
	print "Username: $name Password: $clearpass Name: $realname Klasse: $class\n";
	
}


close LISTE;
close OUTFILE;

chmod (0600, $outfile);
